/*
 *  nvj - NattoView/Java 
 *  (c) 2005-2007 Hidekazu Shiozawa
 *  http://vilab.org
 */

import java.util.*;
import java.util.concurrent.*;
import java.awt.event.*;
import javax.swing.Timer;

public abstract class NvChart {

    protected Map<Object, NvNode> nodes;
    protected NvDrawer drawer;
    protected NvLayout layouter;
    protected Comparator<NvNode> comparator;

    protected NvNode root;
    protected NvNode focus;
    
    protected int dimension = 0;
    private Timer timer = null;

    private class TimerListener implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            animation();
        }
    }
    
    public NvChart(NvLayout layout) {
        this.nodes = new ConcurrentHashMap<Object, NvNode>();
        this.layouter = layout;
    }

    public void setDrawer(NvDrawer view) {
        this.drawer = view;
        timer = new javax.swing.Timer(200, new TimerListener());
        timer.start();
    }

    public NvNode findNode(Object id) {
        return nodes.get(id);
    }

    public NvNode addNode(Object id, String label) {
        NvNode node = findNode(id);
        if (node == null) {
            node = createNode(id, label);
            nodes.put(id, node);
            if (nodes.size() == 1) {
                root = node;
                setFocus(node);
            }
        }
        System.err.print(node);
        System.err.println(" (" + nodeNumbers() + ")");
        return node;
    }

    public NvLink addLink(NvNode source, NvNode dest, String label) {
        NvLink link = source.findLink(dest);
        if (link == null) {
            link = createLink(source, dest, label);
            source.addLink(link);
            dest.addRevLink(link);
            /*
            if (dest != root && dest.revLinkNumbers() == 1) {
                dest.parent = source;
                for (NvNode node = source; node != null; node = node.parent) {
                    System.err.println(node);
                    node.descendants++;
                }
            }
            */
        }
        System.err.println(link);
        return link;
    }

    public int gridDimension() {
        if (dimension == 0) {
            dimension = layouter.layout(this);
        }
        return dimension;
    }

    public void layout() {
        dimension = layouter.layout(this);
        drawer.update();
    }

    public void animation() {
        if (layouter.animation(this))
            drawer.update();
    }

    public Collection<NvNode> getNodes() {
        return nodes.values();
    }

    public Iterator<NvNode> nodeIterator() {
        return nodes.values().iterator();
    }

    public int nodeNumbers() {
        return nodes.values().size();
    }

    public void setFocus(NvNode node) {
        focus = node;
        if (node.renderingMode != NvNode.THUMBNAIL) {
            startFetchingThumbnail(node);
        }
        node.changed = true;
        drawer.setTitle("nvj - " + node.getLabel(0));
        drawer.setSliderLift(5 - node.doi);
    }

    public NvNode getFocus() {
        return focus;
    }

    public void liftUp(NvNode node) {
        liftUp(node, node.doi + 1);
        layout();
        drawer.update();
    }

    public void liftUp(NvNode node, int doi) {
        node.doi = doi;
        for (NvLink link: node.getLinks()) {
            NvNode dest = link.getDest();
            if (dest.doi < doi - 1)
                liftUp(dest, doi - 1);
        }
    }

    public void liftDown(NvNode node) {
        liftDown(node, node.doi - 1);
        layout();
        drawer.update();
    }

    public void liftDown(NvNode node, int doi) {
        if (doi < 0)
            return;
        node.doi = doi;
        for (NvLink link: node.getLinks()) {
            NvNode dest = link.getDest();
            if (dest.doi > 0 && dest.doi > doi)
                liftDown(dest, doi);
        }
    }

    public void liftDownAll() {
        for (NvNode node: getNodes()) {
            node.doi = 0;
        }
        drawer.update();
    }

    protected abstract NvNode createNode(Object id, String label);
    protected abstract NvLink createLink(NvNode soruce, NvNode dest, String label);

    public abstract void startScanningLinks(NvNode node);
    public abstract void startFetchingThumbnail(NvNode node);
}
