/*
 *  nvj - NattoView/Java 
 *  (c) 2005-2007 Hidekazu Shiozawa
 *  http://vilab.org
 */

import java.awt.*;
import java.awt.event.*;
import java.awt.font.*;
import java.awt.image.*;
import java.text.*;
import javax.swing.*;

public abstract class NvDrawer implements KeyListener, MouseListener {

    public JFrame frame;
    protected NvChart chart;
    protected JScrollBar sb_xt, sb_xr, sb_yt, sb_yr, sb_z, sb_lift;
    protected int xgrid = 64, ygrid = 32;

    static public Color backColor = new Color(1.0f, 0.95f, 0.95f, 1.0f);
    static public Color floorColor = new Color(1.0f, 1.0f, 0.9f, 1.0f);
    static public Color gridColor = new Color(0.9f, 0.8f, 0.8f, 1.0f);
    static public Color shadowColor = new Color(0.9f, 0.8f, 0.8f, 0.5f);
    static public Color nodeColor = new Color(1.0f, 1.0f, 0.9f, 1.0f);
    static public Color nodeColor2 = new Color(0.8f, 0.8f, 1.0f, 1.0f);
    static public Color linkColor = new Color(0.8f, 0.8f, 0.8f, 0.5f);
    static public Color linkColor2 = new Color(0.5f, 0.9f, 0.5f, 0.5f);
    static public Color labelColor = Color.black;

    public NvDrawer(NvChart chart, int xgrid, int ygrid) {
        this.chart = chart;
        this.xgrid = xgrid;
        this.ygrid = ygrid;
        frame = new JFrame();
        setTitle("nvj");
        frame.setBounds(0, 0, 650, 650);
        frame.setVisible(true);
        
        sb_xt = new JScrollBar(JScrollBar.HORIZONTAL, 0, 0, -50, 50);
        sb_xt.addAdjustmentListener(new AdjustmentListener() {
            public void adjustmentValueChanged(AdjustmentEvent e) {
                sliderChangedXT(e.getValue());
            }
        });
        sb_xr = new JScrollBar(JScrollBar.HORIZONTAL, 0, 0, -90, 90);
        sb_xr.addAdjustmentListener(new AdjustmentListener() {
            public void adjustmentValueChanged(AdjustmentEvent e) {
                sliderChangedXR(e.getValue());
            }
        });
        sb_yt = new JScrollBar(JScrollBar.VERTICAL, 0, 0, -50, 50);
        sb_yt.addAdjustmentListener(new AdjustmentListener() {
            public void adjustmentValueChanged(AdjustmentEvent e) {
                sliderChangedYT(e.getValue());
            }
        });
        sb_yr = new JScrollBar(JScrollBar.VERTICAL, 0, 0, -90, 0);
        sb_yr.addAdjustmentListener(new AdjustmentListener() {
            public void adjustmentValueChanged(AdjustmentEvent e) {
                sliderChangedYR(e.getValue());
            }
        });
        sb_z = new JScrollBar(JScrollBar.VERTICAL, -20, 0, -100, 0);
        sb_z.addAdjustmentListener(new AdjustmentListener() {
            public void adjustmentValueChanged(AdjustmentEvent e) {
                sliderChangedZ(e.getValue());
            }
        });
        sb_lift = new JScrollBar(JScrollBar.VERTICAL, 5, 0, 0, 5);
        sb_lift.addAdjustmentListener(new AdjustmentListener() {
            public void adjustmentValueChanged(AdjustmentEvent e) {
                sliderChangedLift(e.getValue());
            }
        });

        JPanel spanel = new JPanel();
        frame.getContentPane().add(spanel, BorderLayout.SOUTH);
        spanel.setLayout(new GridLayout(1, 2));
        spanel.add(sb_xt);
        spanel.add(sb_xr);
                
        JPanel epanel = new JPanel();
        frame.getContentPane().add(epanel, BorderLayout.EAST);
        epanel.setLayout(new GridLayout(2, 1));                
        epanel.add(sb_yt);
        epanel.add(sb_yr);        

        JPanel wpanel = new JPanel();
        frame.getContentPane().add(wpanel, BorderLayout.WEST);
        wpanel.setLayout(new GridLayout(2, 1));                
        wpanel.add(sb_lift);
        wpanel.add(sb_z);        
    }
    
    public abstract void update();

    public abstract void initDrawer() throws Exception;

    class ThumbnailObserver implements ImageObserver {
        public boolean imageUpdate(Image image, int info, int x, int y, int w, int h) {
            if ((info & ALLBITS) == 0) return true;
            update();
            return false;
        }
    }
    ImageObserver imageObserver = new ThumbnailObserver();

    public BufferedImage makeLabelImage(String str, int width, int height, boolean flag) {
        BufferedImage bi = new BufferedImage(width, height,
                BufferedImage.TYPE_4BYTE_ABGR);
        Graphics2D g2 = (Graphics2D)bi.getGraphics();
        if (flag) g2.setColor(nodeColor2);
        else      g2.setColor(nodeColor);
        g2.fillRect(0, 0, bi.getWidth(), bi.getHeight());
        // System.err.println(str);

        g2.setColor(labelColor);
        FontRenderContext frc = g2.getFontRenderContext();
        AttributedString astr = new AttributedString(str);
        AttributedCharacterIterator iter = astr.getIterator();
        LineBreakMeasurer measurer = new LineBreakMeasurer(iter, frc);
        float x = 1, y = 0;
        while (measurer.getPosition() < str.length()) {
            TextLayout layout = measurer.nextLayout(bi.getWidth() - 2);
            y += layout.getAscent();
            if (y > bi.getHeight())
                break;
            layout.draw(g2, x, y);
            y += layout.getDescent() + layout.getLeading();
        }
        g2.drawRoundRect(0, 0, bi.getWidth() - 1, bi.getHeight() - 1, 8, 8);
        g2.dispose();
        return bi;
    }
    
    void setTitle(String title) {
        frame.setTitle(title);
    }

    public void keyPressed(KeyEvent e) {}
    public void keyReleased(KeyEvent e) {}
    
    public void mouseReleased(MouseEvent e) {}
    public void mouseClicked(MouseEvent e) {}
    public void mouseEntered(MouseEvent e) {}
    public void mouseExited(MouseEvent e) {}

    public void sliderChangedXT(int v) {}
    public void sliderChangedXR(int v) {}
    public void sliderChangedYT(int v) {}
    public void sliderChangedYR(int v) {}
    public void sliderChangedZ(int v) {}
    public void sliderChangedLift(int v) {}

    public void setSliderXT(int v) {}
    public void setSliderXR(int v) {}
    public void setSliderYT(int v) {}
    public void setSliderYR(int v) {}
    public void setSliderZ(int v) {}
    public void setSliderLift(int v) {
        sb_lift.setValue(v);
    }
}
