/*
 *  nvj - NattoView/Java 
 *  (c) 2005-2007 Hidekazu Shiozawa
 *  http://vilab.org
 */

import java.util.*;
import java.util.concurrent.*;
import java.awt.Image;

public abstract class NvNode {

    protected NvChart chart;
    protected Object id;
    protected Map<NvNode, NvLink> links;
    protected Map<NvNode, NvLink> revLinks;
    protected String label;
    public int doi = 0;

    public final static int LABEL = 0, THUMBNAIL = 1;
    protected int renderingMode = LABEL;
    protected Object renderingCache = null;
    protected Image thumbnail = null; // temporal

    public float x = 0, y = 0, z = 0;
    public float xTarget = 0, yTarget = 0, zTarget = 0;
    public float fx, fy, fz;
    public long lastScan = -1;
    public boolean changed = false;
    public boolean visible = true;

    public NvNode parent = null;
    public int descendants = 0;
    
    public Object pickingId;
    // public Object content;

    public NvNode(NvChart chart, Object id, String label) {
        this.links = new ConcurrentHashMap<NvNode, NvLink>();
        this.revLinks = new ConcurrentHashMap<NvNode, NvLink>();
        this.chart = chart;
        this.id = id;
        this.label = label;
    }

    public NvChart getChart() {
        return chart;
    }

    public Object getId() {
        return id;
    }

    public String getLabel(int maxlen) {
        String l = label;
        if (l == null)
            l = id.toString();
        if (maxlen != 0 && l.length() > maxlen)
            l = l.substring(0, maxlen);
        return l;
    }

    public void setLabel(String label) {
        this.label = label;
    }

    public NvLink findLink(NvNode node) {
        return links.get(node);
    }

    public NvLink findRevLink(NvNode node) {
        return revLinks.get(node);
    }

    protected void addLink(NvLink link) {
        links.put(link.getDest(), link);
    }

    protected void addRevLink(NvLink link) {
        revLinks.put(link.getSource(), link);
    }

    public Collection<NvLink> getLinks() {
        return links.values();
    }

    public Collection<NvLink> getRevLinks() {
        return revLinks.values();
    }
    /*
    public Iterator<NvLink> linkIterator() {
        return links.values().iterator();
    }

    public Iterator<NvLink> revLinkIterator() {
        return revLinks.iterator();
    }
    */
    public int linkNumbers() {
        return links.values().size();
    }

    public int revLinkNumbers() {
        return revLinks.values().size();
    }

    public boolean isSource(NvLink link) {
        return links.containsValue(link);
    }

    public boolean isDest(NvLink link) {
        return revLinks.containsValue(link);
    }

    public String toString() {
        return "[Node] " + label + ": " + id.toString();
    }
}
