/*
 *  nvj - NattoView/Java 
 *  (c) 2005-2007 Hidekazu Shiozawa
 *  http://vilab.org
 */

import java.io.*;
import java.net.*;
import java.awt.*;
import javax.swing.text.*;
import javax.swing.text.html.*;
import javax.swing.text.html.parser.*;

public class WebChart extends NvChart {

    // static final String CrenaHtml2jpg = "CrenaHtml2jpg.exe -fjpeg -q92 -t25";
    static final String CrenaHtml2jpg = "CrenaHtml2jpg.exe -fjpeg -q92 -t120";
    static String thumbnailFile = NvSettings.tmpDir
        + System.getProperty("file.separator") + "nvj.jpg";

    public WebChart(NvLayout layout) {
        super(layout);
        this.comparator = new WebNodeComparator();
    }

    protected NvNode createNode(Object id, String label) {
        WebNode node = new WebNode(this, (URL)id, label);
        return node;
    }

    protected NvLink createLink(NvNode source, NvNode dest, String label) {
        return new WebLink((WebNode)source, (WebNode)dest, label);
    }

    public void startScanningLinks(NvNode node) {
        LinkScanner scanner = new LinkScanner((WebNode)node);
        new Thread(scanner).start();
    }

    class LinkScanner implements Runnable {
        WebNode source;
        
        LinkScanner(WebNode node) {
            this.source = node;
        }
        
        public void run() {
            source.lastScan = System.currentTimeMillis();
            try {
                final URL url = new URL(source.getId().toString());
                HttpURLConnection conn = (HttpURLConnection)url.openConnection();
                conn.setRequestMethod("GET");
                conn.setRequestProperty("User-Agent", NvSettings.userAgent);
                String ctype = conn.getContentType();

                String encoding = "JISAutoDetect";
                if (ctype != null && ctype.indexOf("utf-8") > 0) {
                    encoding = "UTF8";
                }
                BufferedReader in = new BufferedReader(new InputStreamReader(
                        conn.getInputStream(), encoding));
                System.err.println("" + url + " : analyzing...");

                WebParserCallback parserCallback = new WebParserCallback(source, url);
                new ParserDelegator().parse(in, parserCallback, true);

                in.close();
                conn.disconnect();
                System.err.println("" + url + " : done!");

            } catch (MalformedURLException e) {
                e.printStackTrace();
            } catch (IOException e) {
                e.printStackTrace();
            }
            layout();
        }
    }

    class WebParserCallback extends HTMLEditorKit.ParserCallback {
        WebNode source;
        URL base;
        URL href = null;
        
        StringBuffer text;
        // StringBuffer content = new StringBuffer();

        WebParserCallback(WebNode node, URL base) {
            this.source = node;
            this.base = base;
        }
        
        public void handleStartTag(HTML.Tag tag,
                MutableAttributeSet aset, int pos) {
            if (tag.equals(HTML.Tag.A)) {
                try {
                    href = new URL(base, (String)aset
                            .getAttribute(HTML.Attribute.HREF));
                    if (href.getProtocol().equals("http")) {
                        text = new StringBuffer();
                    } else {
                        href = null;
                    }
                } catch (Exception e) {
                    href = null;
                }
            } else if (tag.equals(HTML.Tag.TITLE)) {
                text = new StringBuffer();
            }
        }

        public void handleEndTag(HTML.Tag tag, int pos) {
            if (tag.equals(HTML.Tag.A) && href != null) {
                href = WebNode.urlToId(href);
                System.err.println(href);
                if (!href.toString().equals(source.getId().toString())) {
                    WebNode node = (WebNode)findNode(href);
                    if (node == null) {
                        node = (WebNode)addNode(href, text.toString());
                        layouter.locateNewNode(node, source);
                    } else if (node.lastScan < 0) {
                        node.setLabel(text.toString());
                    }
                    if (node.z < source.doi - 1)
                        liftUp(node, source.doi - 1);
                    if (source != node)
                        addLink(source, node, text.toString());
                }
                href = null;
                text = null;
            } else if (tag.equals(HTML.Tag.TITLE) && text != null) {
                // System.err.println(text);
                source.setLabel(text.toString());
                text = null;
            }
        }

        public void handleSimpleTag(HTML.Tag tag,
                MutableAttributeSet aset, int pos) {
            if (tag.equals(HTML.Tag.BASE)) {
                try {
                    base = new URL(base, (String)aset
                            .getAttribute(HTML.Attribute.HREF));
                } catch (MalformedURLException e) {
                    ;
                }
            } else if (tag.equals(HTML.Tag.IMG)) {
                String alt = (String)aset
                        .getAttribute(HTML.Attribute.ALT);
                if (alt != null) {
                    // content.append(alt);
                    if (text != null)
                        text.append(alt);
                }
            }
        }

        public void handleText(char[] data, int pos) {
            // content.append(data);
            if (text != null)
                text.append(data);
        }

        public void flush() {
            // source.content = content.toString();
            // content = null;
        }
    }
    
    public void startFetchingThumbnail(NvNode node) {
        ThumbnailLoader loader = new ThumbnailLoader((WebNode)node);
        new Thread(loader).start();
    }
    
    class ThumbnailLoader implements Runnable {
        NvNode node;
        
        ThumbnailLoader(WebNode node) {
            this.node = node;
        }
        
        public void run() {
            try {
                String thumbnailer = CrenaHtml2jpg
                    + " -w" + (drawer.xgrid * 6) + "x" + (drawer.ygrid * 6)
                    + " -s" + (drawer.xgrid * 2) + "x" + (drawer.ygrid * 2)
                    + " -u" + ((URL)node.id).toString()
                    + " -o" + thumbnailFile;
                System.err.println(thumbnailer);
                Process process = Runtime.getRuntime().exec(thumbnailer);
                process.waitFor();
                node.thumbnail =
                    Toolkit.getDefaultToolkit().createImage(thumbnailFile);
                process.destroy();
                node.renderingMode = NvNode.THUMBNAIL; 
                // node.renderingCache = null;
                node.changed = true;
                drawer.update();
            } catch (Exception e) {
                ;
            }
        }
    }
}

